//
//  ATKAppDelegate.m
//  ExampleApplication
//
//  Created by Nick Kitchener on 07/01/2013.
//  Copyright (c) 2013 ATIK. All rights reserved.
//

#import "ATKAppDelegate.h"

@implementation ATKAppDelegate

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
    image = [NSImage imageNamed:@"logo.png"];
    
    NSLog(@"Initialising drivers..");
    
    // start up drivers
    _modernDrivers = [[ATIKOSXDrivers alloc]init];
    _legacyDrivers = [[ATIKOSXLegacyDrivers alloc]init];
    
    // If you have 16IC then use the following to add constant ids for each camera serial number
    // [_legacyDrivers registerFTDIChipId:<#(int)#> forUSBSerialNumber:<#(NSString *)#>]
    
    // now we'll set the notification delegate - this object will receive
    // connection and disconnection events (this is optional but useful)
    [_modernDrivers setNotificationDelegate:self];
    [_legacyDrivers setNotificationDelegate:self];
    
    [_modernDrivers startSupport];
    [_legacyDrivers startSupport];
    
    NSLog(@"Initialisation complete");
    
    // by this stage the drivers will have identited all the existing cameras connected
    // and will identify any connected cameras

    // We use the serviceManagement method to get the discovery/management interface
    id<Services> servicesModern = [_modernDrivers serviceManagement];
    id<Services> servicesLegacy = [_legacyDrivers serviceManagement];
    
    // next if we want to find the available services at any point:
    NSDictionary *availableModern = [servicesModern availableServices];
    NSLog(@"The following ATIK services are available: %@", availableModern);

    NSDictionary *availableLegacy = [servicesLegacy availableServices];
    NSLog(@"The following ATIK services are available: %@", availableLegacy);
}

-(void)applicationWillTerminate:(NSNotification *)notification {
    // release any claimed cameras
    if(_titan!=nil) {
        id<Services> services = [_modernDrivers serviceManagement];
        [services releaseService:_titan];
        _titan=nil;
    }
    
    // unhook the drivers
    [_modernDrivers endSupport];
    [_legacyDrivers endSupport];
    
    // release and let ARC release them when ready.
    _modernDrivers = nil;
    _legacyDrivers = nil;
}

-(IBAction)claimCamera:(id)sender {
    // To claim a device - use claimService
    id<Services> services = [_modernDrivers serviceManagement];

    
    // you'll need to put your camera here instead of ARTEMISCCD
    // Your cameras will appears in the debug output window as available services..
    id serviceObject = [services claimService:@"ARTEMISCCD"];
    if( ![serviceObject conformsToProtocol:@protocol(Imager100)]) {
        _titan = nil;
    } else {
        _titan =  serviceObject;
        NSLog(@"Claimed camera %@", [_titan uniqueIdentity]);
    }

    // the _titan object is a fully initalised camera ready for use.
}

-(IBAction)releaseCamera:(id)sender {
    id<Services> services = [_modernDrivers serviceManagement];
    NSLog(@"Releasing %@", [_titan uniqueIdentity]);
    [services releaseService:_titan];
    _titan=nil;
}

-(IBAction)takeExposure:(id)sender {
    NSLog(@"Taking exposure..");
    // create a new image frame
    ImageFrame  *newExposure = [[ImageFrame alloc]initWithSize:0 at:nil];
    
    // set up the parameters - duration, subframe, binning, preview mode etc.
    newExposure.duration = 5.0;
    
    // ask the camera to prepare the image frame
    [_titan prepare:newExposure];
    
    // tell the image frame to self allocate the image buffer
    [newExposure selfAllocate];
    
    // take the exposure immediately
    [_titan snapShot:newExposure blocking:YES];
    
    // ask the device to post process.
    [_titan postProcess:newExposure];
    
    [self replaceWindowImage:newExposure];
    NSLog(@"Taking exposure complete!");
}

-(IBAction)selectFilter:(id)sender {
    // To claim a device - use claimService
    id<Services> services = [_modernDrivers serviceManagement];
    
    id<FilterWheel100>fw=nil;
    
    id serviceObject = [services claimService:@"ATIK EFW2"];
    if( [serviceObject conformsToProtocol:@protocol(FilterWheel100)]) {
        fw = serviceObject;
    }
    
    NSDictionary *availableFilters = [fw availableFilters];
    
    NSLog(@" %i available filters.", availableFilters.count);
    NSLog(@" Available filters %@", availableFilters);
    
    NSLog(@" Selecting random filter");
    
    int selectedFilter = rand()%availableFilters.count;
    
    [fw setPosition:selectedFilter]; // this will block until filter loaded

    NSLog(@"Filter selected");
}

-(void)replaceWindowImage:(ImageFrame*)frame {
    // make an NSBitmapRep form he image..
    
    uint8_t *input = (uint8_t*) frame.imagebuffer;
    
    unsigned char *planes[1];
    planes[0]=(unsigned char*)input;
    
    NSBitmapImageRep *bitmapRep = [[NSBitmapImageRep alloc] initWithBitmapDataPlanes:planes
                                                                          pixelsWide:frame.width
                                                                          pixelsHigh:frame.height
                                                                       bitsPerSample:16
                                                                     samplesPerPixel:1
                                                                            hasAlpha:NO
                                                                            isPlanar:NO
                                                                      colorSpaceName:NSCalibratedWhiteColorSpace
                                                                        bitmapFormat:0
                                                                         bytesPerRow:(2 * frame.width)
                                                                        bitsPerPixel:16];
    
    if( bitmapRep!=nil) {
        NSLog(@"Updating window..");
        NSData *tiff = [bitmapRep representationUsingType:NSTIFFFileType properties:nil];
        //[tiff writeToURL:url atomically:YES];
        image = [[NSImage alloc] initWithSize:NSMakeSize(frame.width, frame.height)];
        [image addRepresentation:bitmapRep];
        [_imageView setImage:image];
    }
}


@end

@implementation ATKAppDelegate (ATIKOSXDriverNotificationDelegate)

-(void)deviceConnected:(NSString*)identity {
    NSLog(@"The following ATIK device attached: %@", identity);
}

-(void)deviceDisconnected:(NSString*)identity {
    NSLog(@"The following ATIK device detached: %@", identity);
}

@end
